@@ -6,6 +6,11 @@ require 'pathname'
 # The containing module for Rouge
 module Rouge
   class << self
+    def reload!
+      Object.send :remove_const, :Rouge
+      load __FILE__
+    end
+
     # Highlight some text with a given lexer and formatter.
     #
     # @example
@@ -67,7 +72,9 @@ load load_dir.join('rouge/themes/thankful_eyes.rb')
 load load_dir.join('rouge/themes/colorful.rb')
 load load_dir.join('rouge/themes/base16.rb')
 load load_dir.join('rouge/themes/github.rb')
+load load_dir.join('rouge/themes/igor_pro.rb')
 load load_dir.join('rouge/themes/monokai.rb')
 load load_dir.join('rouge/themes/molokai.rb')
 load load_dir.join('rouge/themes/monokai_sublime.rb')
 load load_dir.join('rouge/themes/gruvbox.rb')
+load load_dir.join('rouge/themes/tulip.rb')
