@@ -11,7 +11,7 @@ module Rouge
       mimetypes 'text/x-matlab', 'application/x-matlab'
 
       def self.analyze_text(text)
-        return 0.4 if text.match(/^\s*% /) # % comments are a dead giveaway
+        return 0.4 if text =~ /^\s*% / # % comments are a dead giveaway
       end
 
       def self.keywords
@@ -53,7 +53,7 @@ module Rouge
 
         rule %r{[(){};:,\/\\\]\[]}, Punctuation
 
-        rule /~=|==|<<|>>|[-~+\/*%=<>&^|.]/, Operator
+        rule /~=|==|<<|>>|[-~+\/*%=<>&^|.@]/, Operator
 
 
         rule /(\d+\.\d*|\d*\.\d+)(e[+-]?[0-9]+)?/i, Num::Float
@@ -61,7 +61,8 @@ module Rouge
         rule /\d+L/, Num::Integer::Long
         rule /\d+/, Num::Integer
 
-        rule /'/, Str::Single, :string
+        rule /'(?=(.*'))/, Str::Single, :string
+        rule /'/, Operator
       end
 
       state :string do
