@@ -13,16 +13,18 @@ module Rouge
                 '*.module', '*.inc', '*.profile', '*.install', '*.test'
       mimetypes 'text/x-php'
 
-      default_options :parent => 'html'
+      option :start_inline, 'Whether to start with inline php or require <?php ... ?>. (default: best guess)'
+      option :funcnamehighlighting, 'Whether to highlight builtin functions (default: true)'
+      option :disabledmodules, 'Disable certain modules from being highlighted as builtins (default: empty)'
+
+      def initialize(*)
+        super
 
-      def initialize(opts={})
         # if truthy, the lexer starts highlighting with php code
         # (no <?php required)
-        @start_inline = opts.delete(:start_inline)
-        @funcnamehighlighting = opts.delete(:funcnamehighlighting) { true }
-        @disabledmodules = opts.delete(:disabledmodules) { [] }
-
-        super(opts)
+        @start_inline = bool_option(:start_inline) { :guess }
+        @funcnamehighlighting = bool_option(:funcnamehighlighting) { true }
+        @disabledmodules = list_option(:disabledmodules)
       end
 
       def self.builtins
@@ -41,12 +43,22 @@ module Rouge
         end
       end
 
-      def start_inline?
-        !!@start_inline
-      end
+      # source: http://php.net/manual/en/language.variables.basics.php
+      # the given regex is invalid utf8, so... we're using the unicode
+      # "Letter" property instead.
+      id = /[\p{L}_][\p{L}\p{N}_]*/
+      nsid = /#{id}(?:\\#{id})*/
 
       start do
-        push :php if start_inline?
+        case @start_inline
+        when true
+          push :template
+          push :php
+        when false
+          push :template
+        when :guess
+          # pass
+        end
       end
 
       def self.keywords
@@ -70,6 +82,14 @@ module Rouge
       end
 
       state :root do
+        # some extremely rough heuristics to decide whether to start inline or not
+        rule(/\s*(?=<)/m) { delegate parent; push :template }
+        rule(/[^$]+(?=<\?(php|=))/) { delegate parent; push :template }
+
+        rule(//) { push :template; push :php }
+      end
+
+      state :template do
         rule /<\?(php|=)?/, Comment::Preproc, :php
         rule(/.*?(?=<\?)|.*/m) { delegate parent }
       end
@@ -77,7 +97,7 @@ module Rouge
       state :php do
         rule /\?>/, Comment::Preproc, :pop!
         # heredocs
-        rule /<<<('?)([a-z_]\w*)\1\n.*?\n\2;?\n/im, Str::Heredoc
+        rule /<<<('?)(#{id})\1\n.*?\n\2;?\n/im, Str::Heredoc
         rule /\s+/, Text
         rule /#.*?\n/, Comment::Single
         rule %r(//.*?\n), Comment::Single
@@ -85,12 +105,12 @@ module Rouge
         rule %r(/\*\*/), Comment::Multiline
         rule %r(/\*\*.*?\*/)m, Str::Doc
         rule %r(/\*.*?\*/)m, Comment::Multiline
-        rule /(->|::)(\s*)([a-zA-Z_][a-zA-Z0-9_]*)/ do
+        rule /(->|::)(\s*)(#{id})/ do
           groups Operator, Text, Name::Attribute
         end
 
         rule /[~!%^&*+=\|:.<>\/?@-]+/, Operator
-        rule /[\[\]{}();,]+/, Punctuation
+        rule /[\[\]{}();,]/, Punctuation
         rule /class\b/, Keyword, :classname
         # anonymous functions
         rule /(function)(\s*)(?=\()/ do
@@ -103,16 +123,16 @@ module Rouge
           push :funcname
         end
 
-        rule /(const)(\s+)([a-zA-Z_]\w*)/i do
+        rule /(const)(\s+)(#{id})/i do
           groups Keyword, Text, Name::Constant
         end
 
         rule /(true|false|null)\b/, Keyword::Constant
-        rule /\$\{\$+[a-z_]\w*\}/i, Name::Variable
-        rule /\$+[a-z_]\w*/i, Name::Variable
+        rule /\$\{\$+#{id}\}/i, Name::Variable
+        rule /\$+#{id}/i, Name::Variable
 
         # may be intercepted for builtin highlighting
-        rule /[\\a-z_][\\\w]*/i do |m|
+        rule /\\?#{nsid}/i do |m|
           name = m[0]
 
           if self.class.keywords.include? name
@@ -136,11 +156,11 @@ module Rouge
 
       state :classname do
         rule /\s+/, Text
-        rule /[a-z_][\\\w]*/i, Name::Class, :pop!
+        rule /#{nsid}/, Name::Class, :pop!
       end
 
       state :funcname do
-        rule /[a-z_]\w*/i, Name::Function, :pop!
+        rule /#{id}/, Name::Function, :pop!
       end
 
       state :string do
@@ -148,7 +168,7 @@ module Rouge
         rule /[^\\{$"]+/, Str::Double
         rule /\\([nrt\"$\\]|[0-7]{1,3}|x[0-9A-Fa-f]{1,2})/,
           Str::Escape
-        rule /\$[a-zA-Z_][a-zA-Z0-9_]*(\[\S+\]|->[a-zA-Z_][a-zA-Z0-9_]*)?/, Name::Variable
+        rule /\$#{id}(\[\S+\]|->#{id})?/, Name::Variable
 
         rule /\{\$\{/, Str::Interpol, :interp_double
         rule /\{(?=\$)/, Str::Interpol, :interp_single
