@@ -37,7 +37,7 @@ module Rouge
         demoOptionKeyPressed demoShiftKeyPressed demoShow demoWaitForInput
         demoWindowTitle demoX demoY differenceLimensToPhon do editor endPause
         endSendPraat endsWith erb erbToHertz erf erfc exitScript exp
-        extractNumber fileReadable fisherP fisherQ floor gaussP gaussQ
+        extractNumber fileReadable fisherP fisherQ floor gaussP gaussQ hash
         hertzToBark hertzToErb hertzToMel hertzToSemitones imax imin
         incompleteBeta incompleteGammaP index index_regex integer invBinomialP
         invBinomialQ invChiSquareQ invFisherQ invGaussQ invSigmoid invStudentQ
@@ -104,12 +104,21 @@ module Rouge
           groups Text, Comment::Single
         end
 
-        rule /^#.*?$/,        Comment::Single
-        rule /;[^\n]*/,       Comment::Single
-        rule /\s+/,           Text
-        rule /\bprocedure\b/, Keyword,        :procedure_definition
-        rule /\bcall\b/,      Keyword,        :procedure_call
-        rule /@/,             Name::Function, :procedure_call
+        rule /^#.*?$/,         Comment::Single
+        rule /;[^\n]*/,        Comment::Single
+        rule /\s+/,            Text
+
+        rule /(\bprocedure)(\s+)/ do
+          groups Keyword, Text
+          push :procedure_definition
+        end
+
+        rule /(\bcall)(\s+)/ do
+          groups Keyword, Text
+          push :procedure_call
+        end
+
+        rule /@/,              Name::Function, :procedure_call
 
         mixin :function_call
 
@@ -142,7 +151,7 @@ module Rouge
       end
 
       state :command do
-        rule /( ?[\w()-]+ ?)/, Keyword
+        rule /( ?([^\s:\.'])+ ?)/, Keyword
         mixin :string_interpolated
 
         rule /\.{3}/ do
@@ -157,36 +166,30 @@ module Rouge
           push :comma_list
         end
 
-        rule /[\s\n]/,    Text, :pop!
+        rule /[\s]/,    Text, :pop!
       end
 
       state :procedure_call do
-        rule /\s+/, Text
+        mixin :string_interpolated
 
-        rule /([\w.]+)(:|\s*\()/ do
-          groups Name::Function, Punctuation
-          pop!
-        end
+        rule /(:|\s*\()/, Punctuation, :pop!
 
-        rule /([\w.]+)/ do
-          token Name::Function
+        rule /'/,            Name::Function
+        rule /[^:\('\s]+/, Name::Function
+
+        rule /(?=\s+)/ do
+          token Text
           pop!
           push :old_arguments
         end
       end
 
       state :procedure_definition do
-        rule /\s/, Text
+        rule /(:|\s*\()/, Punctuation, :pop!
 
-        rule /([\w.]+)(\s*?[(:])/ do
-          groups Name::Function, Text
-          pop!
-        end
+        rule /[^:\(\s]+/, Name::Function
 
-        rule /([\w.]+)([^\n]*)/ do
-          groups Name::Function, Text
-          pop!
-        end
+        rule /(\s+)/, Text, :pop!
       end
 
       state :function_call do
