@@ -3,6 +3,7 @@
 module Rouge
   module Lexers
     class Properties < RegexLexer
+      title ".properties"
       desc '.properties config files for Java'
       tag 'properties'
 
@@ -10,10 +11,10 @@ module Rouge
       mimetypes 'text/x-java-properties'
 
       def self.analyze_text(text)
-        return 0.1 if text =~ /\A\[[\w.]+\]\s*\w+=\w+/
+        return 0.1 if text =~ /\A\[[\w.-]+\]\s*\w+=\w+/
       end
 
-      identifier = /[\w.]+/
+      identifier = /[\w.-]+/
 
       state :basic do
         rule /[!#].*?\n/, Comment
