@@ -58,17 +58,18 @@ module Rouge
         mixin :whitespace
         rule /\s+/, Text
         rule /#\[/ do
-          token Comment::Preproc; push :attribute
+          token Name::Decorator; push :attribute
         end
         rule(//) { pop! }
+        rule /#\s[^\n]*/, Comment::Preproc
       end
 
       state :attribute do
         mixin :whitespace
         mixin :has_literals
-        rule /[(,)=]/, Comment::Preproc
-        rule /\]/, Comment::Preproc, :pop!
-        rule id, Comment::Preproc
+        rule /[(,)=]/, Name::Decorator
+        rule /\]/, Name::Decorator, :pop!
+        rule id, Name::Decorator
       end
 
       state :whitespace do
@@ -86,7 +87,7 @@ module Rouge
         rule %r([=-]>), Keyword
         rule %r(<->), Keyword
         rule /[()\[\]{}|,:;]/, Punctuation
-        rule /[*!@~&+%^<>=-]/, Operator
+        rule /[*!@~&+%^<>=-\?]|\.{2,3}/, Operator
 
         rule /([.]\s*)?#{id}(?=\s*[(])/m, Name::Function
         rule /[.]\s*#{id}/, Name::Property
@@ -98,6 +99,7 @@ module Rouge
         rule /\bmacro_rules!/, Name::Decorator, :macro_rules
         rule /#{id}!/, Name::Decorator, :macro
 
+        rule /'#{id}/, Name::Variable
         rule /#{id}/ do |m|
           name = m[0]
           if self.class.builtins.include? name
