@@ -8,13 +8,15 @@ module Rouge
       mimetypes 'text/x-yaml'
       tag 'yaml'
       aliases 'yml'
+      filenames '*.yaml', '*.yml'
 
       def self.analyze_text(text)
         # look for the %YAML directive
         return 1 if text =~ /\A\s*%YAML/m
       end
 
-      filenames '*.yaml', '*.yml'
+      SPECIAL_VALUES = Regexp.union(%w(true false null))
+
       # NB: Tabs are forbidden in YAML, which is why you see things
       # like /[ ]+/.
 
@@ -63,6 +65,7 @@ module Rouge
 
       def set_indent(match, opts={})
         if indent < @next_indent
+          puts "    yaml: indenting #{indent}/#{@next_indent}" if @debug
           @indent_stack << @next_indent
         end
 
@@ -122,7 +125,10 @@ module Rouge
         end
 
         # block collection indicators
-        rule(/[?:-](?=[ ]|$)/) { |m| token Punctuation::Indicator; set_indent m[0] }
+        rule(/[?:-](?=[ ]|$)/) do |m|
+          set_indent m[0]
+          token Punctuation::Indicator
+        end
 
         # the beginning of a block line
         rule(/[ ]*/) { |m| token Text; continue_indent(m[0]); pop! }
@@ -166,8 +172,8 @@ module Rouge
 
       state :block_nodes do
         # implicit key
-        rule /:(?=\s|$)/ do |m|
-          token Punctuation::Indicator
+        rule /((?:\w[\w -]*)?)(:)(?=\s|$)/ do |m|
+          groups Name::Attribute, Punctuation::Indicator
           set_indent m[0], :implicit => true
         end
 
@@ -233,7 +239,9 @@ module Rouge
           @block_scalar_indent ||= indent_size
 
           if indent_size < dedent_level
-            pop! 2
+            save_indent m[0]
+            pop!
+            push :indentation
           end
         end
 
@@ -329,6 +337,7 @@ module Rouge
         end
 
         rule /[ ]+/, Str
+        rule SPECIAL_VALUES, Name::Constant
         # regular non-whitespace characters
         rule /[^\s:]+/, Str
       end
